//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import KakaoSDKCommon
import KakaoSDKAuth
import KakaoSDKTalk

extension Array where Element : ParamItem {
    func paramItem(key:String) -> ParamItem? {
        return (self.filter { paramItem in
            return paramItem.key == key && paramItem.selected == true
        }).first
    }
    
    func requestParam(key:String) -> KakaoSDKTalk.Order? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .singleSelectableItem(let values):
            return paramItem.selectedIndex
                .map ({ index in
                    return values[index]
                })
                .compactMap({ element in
                    return Order(rawValue: element)
                }).first
        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> KakaoSDKTalk.FriendOrder? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .singleSelectableItem(let values):
            return paramItem.selectedIndex
                .map ({ index in
                    return values[index]
                })
                .compactMap({ element in
                    return FriendOrder(rawValue: element)
                }).first
        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> LaunchMethod? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .singleSelectableItem(let values):
            return paramItem.selectedIndex
                .map ({ index in
                    return values[index]
                })
                .compactMap({ launchMethod in
                    return LaunchMethod(rawValue: launchMethod)
                }).first
        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> [Prompt]? {
        guard let paramItem = paramItem(key:key), paramItem.selected == true else { return nil }
        switch paramItem.value {
        case .multiSelectableItem(let values):
            return paramItem.selectedIndex
                .map ({ index in
                    return values[index]
                })
                .compactMap({ prompt in
                    return Prompt(rawValue: prompt)
                })
        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> Int? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .editableText(let value):
            return Int(value)
        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> String? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .editableText(let value):
            return value
        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> [String]? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .editableText(let value):
            return value.components(separatedBy: ",")

        default:
            break
        }
        return nil
    }
    
    func requestParam(key:String) -> Bool? {
        guard let paramItem = paramItem(key:key) else { return nil }
        
        switch paramItem.value {
        case .editableText(let value):
            return (value as NSString).boolValue
        case .singleSelectableItem(let values):
            return (values.first! as NSString).boolValue
        default:
            break
        }
        return nil
    }
    
    func requestParam(key: String) -> CertType? {
        guard let paramItem = paramItem(key: key) else { return nil }
        
        switch paramItem.value {
        case .singleSelectableItem(let values):
            return paramItem.selectedIndex
                .map({ index in
                    return values[index]
                })
                .compactMap({ certType in
                    return CertType(rawValue: certType)
                }).first
        default:
            break
        }
        
        return nil
    }
}

extension String {
    func widthOfString(usingFont font: UIFont) -> CGFloat {
        let fontAttributes = [NSAttributedString.Key.font: font]
        let size = self.size(withAttributes: fontAttributes)
        return size.width
    }

    func heightOfString(usingFont font: UIFont) -> CGFloat {
        let fontAttributes = [NSAttributedString.Key.font: font]
        let size = self.size(withAttributes: fontAttributes)
        return size.height
    }

    func sizeOfString(usingFont font: UIFont) -> CGSize {
        let fontAttributes = [NSAttributedString.Key.font: font]
        return self.size(withAttributes: fontAttributes)
    }
}

extension UITextField {
    func setLeftPaddingPoints(_ amount:CGFloat){
        let paddingView = UIView(frame: CGRect(x: 0, y: 0, width: amount, height: self.frame.size.height))
        self.leftView = paddingView
        self.leftViewMode = .always
    }
    func setRightPaddingPoints(_ amount:CGFloat) {
        let paddingView = UIView(frame: CGRect(x: 0, y: 0, width: amount, height: self.frame.size.height))
        self.rightView = paddingView
        self.rightViewMode = .always
    }
}

extension String {
    func getLabelWidth(usedFont: UIFont, maxWidth:CGFloat? = nil) -> CGFloat {
        let width : CGFloat = self.size(withAttributes: [NSAttributedString.Key.font : usedFont]).width
        if let maxWidth = maxWidth {
            return (width >= maxWidth) ? maxWidth : width
        }
        else {
            return width
        }
    }
}

extension ParamNaviView {
    func getTextLabelWidth(text:String? = nil, font:UIFont? = nil, maxWidth:CGFloat? = nil) -> CGFloat {
        if let font = font, let width = text?.getLabelWidth(usedFont: font, maxWidth: maxWidth) {
            return width + 2
        }
        else {
            return 0
        }
    }
}
