//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit

class ParamNaviView: UIView {
    var rightButton: UIButton = {
        let button = UIButton(frame: CGRect.zero)
        return button
    }()
    var titleLabel: UILabel = {
        let label = UILabel(frame: CGRect.zero)
        return label
    }()
    var lineView: UIView = {
        let view = UIView(frame: .zero)
        return view
    }()
      
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.backgroundColor = .white
        rightButton.titleLabel?.font = UIFont.systemFont(ofSize: 16, weight: .light)
        rightButton.setTitle("닫기", for: .normal)
        rightButton.setTitleColor(UIColor.black, for: .normal)
        rightButton.setTitleColor(UIColor.gray, for: .highlighted)
        addSubview(rightButton)

        titleLabel.font = UIFont.systemFont(ofSize: 18, weight: .semibold)
        titleLabel.textAlignment = .center
        titleLabel.numberOfLines = 1
        titleLabel.textColor = UIColor.black
        titleLabel.backgroundColor = UIColor.white
        titleLabel.adjustsFontSizeToFitWidth = true
        addSubview(titleLabel)

        lineView.backgroundColor = UIColor.init(red: 0.1, green: 0.1, blue: 0.1, alpha: 0.05)
        addSubview(lineView)
        
        addSubviewsConstraint()
    }
    
    func addSubviewsConstraint() {
        titleLabel.translatesAutoresizingMaskIntoConstraints = false
        titleLabel.centerXAnchor.constraint(equalTo:centerXAnchor).isActive = true
        titleLabel.centerYAnchor.constraint(equalTo:centerYAnchor).isActive = true
        titleLabel.widthAnchor.constraint(equalToConstant: 200).isActive = true
        titleLabel.heightAnchor.constraint(equalToConstant: 21.5).isActive = true
        
        rightButton.translatesAutoresizingMaskIntoConstraints = false
        rightButton.centerYAnchor.constraint(equalTo:centerYAnchor).isActive = true
        rightButton.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -8).isActive = true
        rightButton.widthAnchor.constraint(equalToConstant: 48).isActive = true
        rightButton.heightAnchor.constraint(equalTo: self.heightAnchor).isActive = true
        
        lineView.translatesAutoresizingMaskIntoConstraints = false
        lineView.topAnchor.constraint(equalTo: bottomAnchor, constant: -2).isActive = true
        lineView.leadingAnchor.constraint(equalTo: leadingAnchor).isActive = true
        lineView.trailingAnchor.constraint(equalTo: trailingAnchor).isActive = true
        lineView.heightAnchor.constraint(equalToConstant: 2).isActive = true
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
//        updateSubviewsConstraint()
    }
    
    func setTitle(title: String) {
        titleLabel.text = title
        setNeedsLayout()
    }
    
    func setRightButtonTitle(title: String) {
        rightButton.setTitle(title, for: .normal)
    }
    
    func setRightButtonAction(target:Any?, action:Selector) {
        rightButton.addTarget(target, action: action, for: .touchUpInside)
    }
}

//MARK: - ParamViewController
class SDKSampleParamViewController: UIViewController, UITableViewDelegate, UITableViewDataSource {
    var backgroundView: UIView!
    var backgroudViewWidthConstraint : NSLayoutConstraint!
    var backgroudViewHeightConstraint : NSLayoutConstraint!
    
    let naviView: ParamNaviView = {
        let naviView = ParamNaviView(frame: CGRect.zero)
        return naviView
    }()
    let tableView: UITableView = {
        let tableView = UITableView(frame: CGRect.zero, style: .plain)
        return tableView
    }()
    let sendButton: UIButton = {
        let button = UIButton(frame: CGRect.zero)
        return button
    }()
    var lineView: UIView = {
        let lineView = UIView(frame: .zero)
        return lineView
    }()
    
    var safeAreaLayoutGuide : UILayoutGuide!
    var sections: [SectionOfRow]?
    
    var apiParams: ApiParams!
    var completionDelay: CGFloat? = nil
    var paramViewCompletionHandler: ((ApiParams) -> Void)?
    
    var viewTitle: String?
    var paramItems: [ParamItem]?
    
    convenience init(apiParams:ApiParams,
                            completionDelay:CGFloat? = nil,
                            paramViewCompletionHandler: ((ApiParams) -> Void)? = nil) {        
        self.init()
        self.apiParams = apiParams
        self.completionDelay = completionDelay
        self.viewTitle = apiParams.name
        self.paramItems = apiParams.paramItems
        self.paramViewCompletionHandler = paramViewCompletionHandler
    }
    
    init() {
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("Never will happen")
    }
    
    override func loadView() {
        super.loadView()
        self.safeAreaLayoutGuide = view.layoutMarginsGuide
        
        setupViews()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if #available(iOS 15.0, *) {
            let navBarAppearance = UINavigationBarAppearance()
            navBarAppearance.configureWithOpaqueBackground()
            //Configure additional customizations here
            UINavigationBar.appearance().standardAppearance = navBarAppearance
            UINavigationBar.appearance().scrollEdgeAppearance = navBarAppearance
        }
        
        extendedLayoutIncludesOpaqueBars = true
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.naviView.setTitle(title: "\(self.viewTitle ?? "")")
    }
    
    func setupViews() {
        backgroundView = UIView(frame: view.frame)
        backgroundView.backgroundColor = UIColor.white
        backgroundView.layer.cornerRadius = CommonConstants.paramUIRadius
        
//        naviView.setLeftButtonAction(target: self, action: #selector(closeButtonAction))
        naviView.setRightButtonAction(target: self, action: #selector(closeButtonAction))
        naviView.layer.cornerRadius = CommonConstants.paramUIRadius
        naviView.layer.masksToBounds = true
        naviView.layer.maskedCorners = [.layerMinXMinYCorner, .layerMaxXMinYCorner]
        
        backgroundView.addSubview(naviView)
        
        tableView.backgroundColor = .lightGray
        tableView.separatorStyle = UITableViewCell.SeparatorStyle.none
        backgroundView.addSubview(tableView)
        if #available(iOS 15, *) {
            tableView.sectionHeaderTopPadding = 0
        }
//        tableView.allowsMultipleSelection = true
//        tableView.allowsSelection = false
        tableView.register(EditableTextParamTableViewCell.self, forCellReuseIdentifier: "EditableTextParamTableViewCell")
        tableView.register(SelectableItemParamTableViewCell.self, forCellReuseIdentifier: "SelectableItemParamTableViewCell")
        
        sendButton.titleLabel?.font = UIFont.systemFont(ofSize: 18, weight: .semibold)
        sendButton.backgroundColor = UIColor.white
        sendButton.setTitle("Request", for: .normal)
        sendButton.setTitleColor(UIColor.black, for: .normal)
        sendButton.setTitleColor(UIColor.gray, for: .highlighted)
        sendButton.layer.cornerRadius = CommonConstants.paramUIRadius
        sendButton.layer.masksToBounds = true
        sendButton.layer.maskedCorners = [.layerMinXMaxYCorner, .layerMaxXMaxYCorner]
        sendButton.addTarget(self, action: #selector(okButtonAction), for: .touchUpInside)
        backgroundView.addSubview(sendButton)

        lineView.backgroundColor = UIColor.init(red: 0.1, green: 0.1, blue: 0.1, alpha: 0.05)
        backgroundView.addSubview(lineView)
        
        view.backgroundColor = UIColor.black.withAlphaComponent(0.7)
        view.addSubview(backgroundView)
        
        addSubviewsConstraint()
    }
    
    func addSubviewsConstraint() {
        let size = calculateBackgroundSize()
        backgroundView.translatesAutoresizingMaskIntoConstraints = false
        backgroudViewWidthConstraint = backgroundView.widthAnchor.constraint(equalToConstant: size.width)
        backgroudViewWidthConstraint.isActive = true
        backgroudViewHeightConstraint = backgroundView.heightAnchor.constraint(equalToConstant: size.height)
        backgroudViewHeightConstraint.isActive = true

        backgroundView.centerXAnchor.constraint(equalTo:view.safeAreaLayoutGuide.centerXAnchor).isActive = true
        backgroundView.centerYAnchor.constraint(equalTo:view.safeAreaLayoutGuide.centerYAnchor).isActive = true
        
        naviView.translatesAutoresizingMaskIntoConstraints = false
        naviView.topAnchor.constraint(equalTo:backgroundView.topAnchor).isActive = true
        naviView.leadingAnchor.constraint(equalTo: backgroundView.leadingAnchor).isActive = true
        naviView.trailingAnchor.constraint(equalTo: backgroundView.trailingAnchor).isActive = true
        naviView.heightAnchor.constraint(equalToConstant: getNaviViewHeight()).isActive = true
        
        tableView.translatesAutoresizingMaskIntoConstraints = false
        tableView.topAnchor.constraint(equalTo: naviView.bottomAnchor).isActive = true
        tableView.leadingAnchor.constraint(equalTo: backgroundView.leadingAnchor).isActive = true
        tableView.trailingAnchor.constraint(equalTo: backgroundView.trailingAnchor).isActive = true
        tableView.heightAnchor.constraint(equalToConstant: getTableViewHeight()).isActive = true
//        tableView.bottomAnchor.constraint(equalTo: sendButton.topAnchor).isActive = true
        
        sendButton.translatesAutoresizingMaskIntoConstraints = false
        sendButton.topAnchor.constraint(equalTo: tableView.bottomAnchor).isActive = true
        sendButton.leadingAnchor.constraint(equalTo: backgroundView.leadingAnchor).isActive = true
        sendButton.trailingAnchor.constraint(equalTo: backgroundView.trailingAnchor).isActive = true
        sendButton.heightAnchor.constraint(equalToConstant: getSendButtonHeight()).isActive = true
        
        lineView.translatesAutoresizingMaskIntoConstraints = false
        lineView.topAnchor.constraint(equalTo: sendButton.topAnchor).isActive = true
        lineView.leadingAnchor.constraint(equalTo: backgroundView.leadingAnchor).isActive = true
        lineView.trailingAnchor.constraint(equalTo: backgroundView.trailingAnchor).isActive = true
        lineView.heightAnchor.constraint(equalToConstant: 2).isActive = true
    }
    
    @objc func okButtonAction(sender: Any?) {
        if let completionHandler = paramViewCompletionHandler {            
            self.dismiss(animated: false) {
                if let completionDelay = self.completionDelay {
                    DispatchQueue.main.asyncAfter(deadline: .now() + completionDelay) {
                        completionHandler(self.apiParams)
                    }
                }
                else {
                    completionHandler(self.apiParams)
                }
            }
        }
    }
    
    @objc func closeButtonAction(sender: Any?) {
        dismiss(animated: false) { }
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func getNaviViewHeight() -> CGFloat {
        var height: CGFloat = CommonConstants.paramViewTitleHeight
        if (UIDevice.current.orientation.isLandscape) {
            height = 32
        }
        return height
    }
    
    func getTableViewHeight() -> CGFloat {
        let cellAreaHeight = CGFloat(apiParams.paramItems.count) * CommonConstants.paramCellHeight
        return cellAreaHeight
    }
    
    func getSendButtonHeight() -> CGFloat {
        return CommonConstants.paramViewRequestButtoneHeight
    }
    
    func getPopupMargine() -> CGFloat {
        var margin  : CGFloat = 30
        if (UIDevice.current.orientation.isLandscape) {
            margin = 20
        }
        return margin
    }
    
    func calculateBackgroundSize() -> CGSize {
        var safeAreaLayoutFrame : CGRect!
        if let root = UIApplication.getMostTopViewController() {
            if #available(iOS 11, *) {
                safeAreaLayoutFrame = root.view.safeAreaLayoutGuide.layoutFrame
            }
            else {
                safeAreaLayoutFrame = view.bounds
            }
        }
        else {
            safeAreaLayoutFrame = view.bounds
        }
        
//        print("safeAreaLayoutFrame: \(String(describing: safeAreaLayoutFrame))")
        let popupMargine = getPopupMargine()
        
        let width = (safeAreaLayoutFrame.width - (popupMargine * 2)) >  CommonConstants.maxWidth ? CommonConstants.maxWidth : safeAreaLayoutFrame.width - (popupMargine * 2)
        let height = (safeAreaLayoutFrame.height - (popupMargine * 2)) > CommonConstants.maxHeight ? CommonConstants.maxHeight : safeAreaLayoutFrame.height - (popupMargine * 2)
        
        let actualHeight = getNaviViewHeight() + getTableViewHeight() + getSendButtonHeight()
        
//        print("view width:\(self.view.frame.width) height:\(self.view.frame.height) actureHeight:\(actualHeight)")
//        print("width:\(width) height:\(height)")
        
        return CGSize(width: width, height: min(actualHeight, height))
    }
    

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if let items = self.paramItems {
            return items.count
        }
        else {
            return 0
        }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let paramItem = self.paramItems![indexPath.row]
        var cell : ParamTableViewCell
        switch paramItem.value {
        case .editableText(_):
            cell = tableView.dequeueReusableCell(withIdentifier: "EditableTextParamTableViewCell", for: indexPath) as! EditableTextParamTableViewCell
        case .multiSelectableItem(_):
            cell = tableView.dequeueReusableCell(withIdentifier: "SelectableItemParamTableViewCell", for: indexPath) as! SelectableItemParamTableViewCell
            (cell as? SelectableItemParamTableViewCell)?.selectMode = .multi
        case .singleSelectableItem(_):
            cell = tableView.dequeueReusableCell(withIdentifier: "SelectableItemParamTableViewCell", for: indexPath) as! SelectableItemParamTableViewCell
            (cell as? SelectableItemParamTableViewCell)?.selectMode = .single
        @unknown default:
            cell = ParamTableViewCell()
            break
        }
        cell.setParamItem(paramItem)
        cell.updateSwitchUI()
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        //nothing to do
        //개별 셀에서 처리
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return CommonConstants.paramCellHeight
    }
}
